<?php
/*
Plugin Name: Smartphone ClickSort
Description: Replaces "dragSortOrder" fields with numeric text fields so that records can be moved quickly on a small screen. This feature is only enabled on mobile devices such as the iPhone.
Version: 1.00
Requires at least: 2.03
*/

$GLOBALS['SMARTPHONE_CLICKSORT_USER_AGENTS'] = array('iPhone'); // list of strings to check UserAgent against to enable this plugin's functionality


// DON'T UPDATE ANYTHING BELOW THIS LINE

addAction('admin_head',               'smartphoneClickSort_onChangeJavaScript');
addAction('section_preDispatch',      'smartphoneClickSort_section_preDispatch', null, 2);
addFilter('listHeader_displayLabel',  'smartphoneClickSort_listHeader_displayLabel', null, 3);
addFilter('listRow_displayValue',     'smartphoneClickSort_listRow_displayValue', null, 4);
addFilter('listRow_tdAttributes',     'smartphoneClickSort_listRow_tdAttributes', null, 4);


function smartphoneClickSort_is_iPhone() {
  foreach ($GLOBALS['SMARTPHONE_CLICKSORT_USER_AGENTS'] as $userAgent) {
    if (strpos($_SERVER['HTTP_USER_AGENT'], $userAgent) !== false) { return true; }
  }
  return false;
}

function smartphoneClickSort_listHeader_displayLabel($label, $tableName, $fieldname) {
  if (!smartphoneClickSort_is_iPhone()) { return $label; }
  
  if ($fieldname == 'dragSortOrder') {
    return 'Order';
  }
  else {
    return $label;
  }
}

function smartphoneClickSort_listRow_displayValue($displayValue, $tableName, $fieldname, $record) {
  if (!smartphoneClickSort_is_iPhone()) { return $displayValue; }
  static $sequence = 1;
  if (@$fieldname == 'dragSortOrder') {
    $html = '<input type="hidden" class="_recordNum" value="' . $record['num'] . '" name="_recordNum" />';
    $html .= '<div style="text-align:center"><a href="#" onclick="smartphoneClickSort_updateOrder(' . $record['num'] . ', ' . $sequence . '); return false;">[ ' . $sequence . ' ]</a></div>';
    $sequence++;
    return $html;
  }
  return $displayValue;
}

function smartphoneClickSort_listRow_tdAttributes($tdAttributes, $tableName, $fieldname, $record) {
  if (!smartphoneClickSort_is_iPhone()) { return $tdAttributes; }
  if (@$fieldname == 'dragSortOrder') {
    return '';
  }
  return $tdAttributes;
}

function smartphoneClickSort_section_preDispatch($tableName, $action) {
  if (!smartphoneClickSort_is_iPhone()) { return; }
  global $TABLE_PREFIX;
  
  if ($action != 'iPhoneReorder') { return; }

  // check input
  $errors = '';
  if (!@$_REQUEST['num'])                             { $errors .= "No 'num' specified!\n"; }
  if (!@$_REQUEST['oldOrder'])                        { $errors .= "No 'oldOrder' specified!\n"; }
  if (!array_key_exists('newPosition', $_REQUEST))    { $errors .= "No 'newPosition' specified!\n"; }
  if (!@$_REQUEST['oldPosition'])                     { $errors .= "No 'oldPosition' specified!\n"; }
  if (preg_match("/[^\d\,]/", $_REQUEST['oldOrder'])) { $errors .= "'oldOrder' contains invalid chars! ('" .htmlspecialchars($_REQUEST['oldOrder']). "')\n"; }
  if ($errors) { die($errors); }
  
  // get input
  $targetNum  = $_REQUEST['num'];                          // num of record to be moved
  $oldOrder   = preg_split("/\,/", $_REQUEST['oldOrder']); // old record order
  $newPos     = ceil($_REQUEST['newPosition']);
  $oldPos     = ceil($_REQUEST['oldPosition']);
  if ($newPos < 1) { $newPos = 1; }
  if ($newPos > $oldPos) { $newPos++; }
  
  // figure out where we want to move this record to: if newPos is 1..N, we want to move record(num) before record(oldOrder[newPos - 1]), otherwise we want to move record(num) after record(last(oldOrder))
  $refNum = 0;  // which record number
  $relPos = -1; // -1 means before, 1 means after
  if ($newPos <= sizeof($oldOrder)) {
    $refNum = $oldOrder[ $newPos - 1 ];
    $relPos = -1;
  }
  else {
    $refNum = $oldOrder[ sizeof($oldOrder) - 1 ];
    $relPos = 1;
  }
  
  $escapedTableName = mysql_escape( $TABLE_PREFIX . $tableName );
  // ensure there are spaces between dragSortOrders in the database
  mysql_query("SET @count = 0") or die("MySQL Error: ". htmlspecialchars(mysql_error()) . "\n");
  mysql_query("UPDATE `$escapedTableName` SET `dragSortOrder` = (SELECT @count := @count + 10) ORDER BY dragSortOrder") or die("MySQL Error: ". htmlspecialchars(mysql_error()) . "\n");
  
  // get the reference record's dragSortOrder
  $refRecord = mysql_query_fetch_row_array("SELECT dragSortOrder FROM `$escapedTableName` WHERE num = '" . mysql_escape($refNum) . "'");
  $refDragSortOrder = $refRecord[0];
  
  // reorder the target record to be before or after the reference record
  mysql_query("UPDATE `$escapedTableName` SET dragSortOrder = '" . ($refDragSortOrder + $relPos * 5) . "' WHERE num = '" . mysql_escape($targetNum) . "'") or die("MySQL Error: ". htmlspecialchars(mysql_error()) . "\n");
  
  // report success by not printing anything
  exit;
}

function smartphoneClickSort_onChangeJavaScript() {
?>
<script>
  function smartphoneClickSort_updateOrder(num, oldPosition) {
    
    // prompt for new position
    var newPosition = prompt("Move to position:", oldPosition);
    newPosition = parseInt(newPosition);
    if (isNaN(newPosition)) { return false; }
    
    // get old order
    var oldOrder = $('._recordNum').map(function(i,e){return $(e).val()}).get().join(',');
    
    // Save changes via ajax
    $.ajax({
      url: '?',
      type: "POST",
      data: {
        menu: $('#menu').val(),
        action: 'iPhoneReorder',
        num: num,
        oldOrder: oldOrder,
        oldPosition: oldPosition,
        newPosition: newPosition
      },
      error: function(XMLHttpRequest, textStatus, errorThrown){
        alert("There was an error sending the request! (" +XMLHttpRequest['status']+" "+XMLHttpRequest['statusText']+" "+textStatus +" "+errorThrown +")");
      },
      success: function(msg){
        if (msg) { alert("Error: " + msg); }
        else { location.href = '?menu=' + $('#menu').val(); }
      }
    });   
  }
</script>
<?php
}

?>